@@ -16,7 +16,9 @@ CREATE FUNCTION plpgsql_check_function_tb(funcoid regprocedure,
                                        anycompatibletype regtype DEFAULT 'int',
                                        anycompatiblerangetype regtype DEFAULT 'int4range',
                                        without_warnings boolean DEFAULT false,
-                                       all_warnings boolean DEFAULT false)
+                                       all_warnings boolean DEFAULT false,
+                                       use_incomment_options boolean DEFAULT true,
+                                       incomment_options_usage_warning boolean DEFAULT false)
 RETURNS TABLE(functionid regproc,
               lineno int,
               statement text,
@@ -47,7 +49,9 @@ CREATE FUNCTION plpgsql_check_function(funcoid regprocedure,
                                        anycompatibletype regtype DEFAULT 'int',
                                        anycompatiblerangetype regtype DEFAULT 'int4range',
                                        without_warnings boolean DEFAULT false,
-                                       all_warnings boolean DEFAULT false)
+                                       all_warnings boolean DEFAULT false,
+                                       use_incomment_options boolean DEFAULT true,
+                                       incomment_options_usage_warning boolean DEFAULT false)
 RETURNS SETOF text
 AS 'MODULE_PATHNAME','plpgsql_check_function'
 LANGUAGE C;
@@ -67,7 +71,9 @@ CREATE FUNCTION plpgsql_check_function_tb(name text,
                                        anycompatibletype regtype DEFAULT 'int',
                                        anycompatiblerangetype regtype DEFAULT 'int4range',
                                        without_warnings boolean DEFAULT false,
-                                       all_warnings boolean DEFAULT false)
+                                       all_warnings boolean DEFAULT false,
+                                       use_incomment_options boolean DEFAULT true,
+                                       incomment_options_usage_warning boolean DEFAULT false)
 RETURNS TABLE(functionid regproc,
               lineno int,
               statement text,
@@ -98,7 +104,9 @@ CREATE FUNCTION plpgsql_check_function(name text,
                                        anycompatibletype regtype DEFAULT 'int',
                                        anycompatiblerangetype regtype DEFAULT 'int4range',
                                        without_warnings boolean DEFAULT false,
-                                       all_warnings boolean DEFAULT false)
+                                       all_warnings boolean DEFAULT false,
+                                       use_incomment_options boolean DEFAULT true,
+                                       incomment_options_usage_warning boolean DEFAULT false)
 RETURNS SETOF text
 AS 'MODULE_PATHNAME','plpgsql_check_function_name'
 LANGUAGE C;
