@@ -4,7 +4,7 @@
  *
  *			  enhanced checks for plpgsql functions
  *
- * by Pavel Stehule 2013-2021
+ * by Pavel Stehule 2013-2022
  *
  *-------------------------------------------------------------------------
  *
@@ -36,6 +36,7 @@ PG_MODULE_MAGIC;
 #endif
 
 PLpgSQL_plugin **plpgsql_check_plugin_var_ptr;
+PLpgSQL_plugin *prev_plpgsql_plugin;
 
 static PLpgSQL_plugin plugin_funcs = { plpgsql_check_profiler_func_init,
 									   plpgsql_check_on_func_beg,
@@ -167,6 +168,7 @@ _PG_init(void)
 		LOAD_EXTERNAL_FUNCTION("$libdir/plpgsql", "plpgsql_ns_lookup");
 
 	plpgsql_check_plugin_var_ptr = (PLpgSQL_plugin **) find_rendezvous_variable( "PLpgSQL_plugin");
+	prev_plpgsql_plugin = *plpgsql_check_plugin_var_ptr;
 	*plpgsql_check_plugin_var_ptr = &plugin_funcs;
 
 	DefineCustomBoolVariable("plpgsql_check.regress_test_mode",
@@ -330,7 +332,7 @@ _PG_init(void)
 		 */
 #if PG_VERSION_NUM >= 150000
 
-		prev_shmem_startup_hook = shmem_startup_hook;
+		prev_shmem_request_hook = shmem_request_hook;
 		shmem_request_hook = plpgsql_check_profiler_shmem_request;
 
 #endif
