@@ -4,7 +4,7 @@
  *
  *			  profiler accessories code
  *
- * by Pavel Stehule 2013-2021
+ * by Pavel Stehule 2013-2022
  *
  *-------------------------------------------------------------------------
  */
@@ -222,6 +222,8 @@ typedef struct profiler_info
 
 #endif
 
+	void	   *prev_plugin_info;
+
 } profiler_info;
 
 typedef struct profiler_iterator
@@ -298,7 +300,7 @@ static MemoryContext profiler_queryid_mcxt = NULL;
 static profiler_stack *top_pinfo = NULL;
 static ExprContext *curr_eval_econtext = NULL;
 
-bool plpgsql_check_profiler = true;
+bool plpgsql_check_profiler = false;
 
 needs_fmgr_hook_type	plpgsql_check_next_needs_fmgr_hook = NULL;
 fmgr_hook_type			plpgsql_check_next_fmgr_hook = NULL;
@@ -1338,11 +1340,7 @@ coverage_internal(Oid fnoid, int coverage_type)
 	if (!HeapTupleIsValid(cinfo.proctuple))
 		elog(ERROR, "cache lookup failed for function %u", cinfo.fn_oid);
 
-	plpgsql_check_get_function_info(cinfo.proctuple,
-									&cinfo.rettype,
-									&cinfo.volatility,
-									&cinfo.trigtype,
-									&cinfo.is_procedure);
+	plpgsql_check_get_function_info(&cinfo);
 
 	plpgsql_check_precheck_conditions(&cinfo);
 
@@ -2805,6 +2803,39 @@ plpgsql_check_profiler_func_init(PLpgSQL_execstate *estate, PLpgSQL_function *fu
 {
 	profiler_info *pinfo = NULL;
 
+	if (prev_plpgsql_plugin)
+	{
+		prev_plpgsql_plugin->error_callback = (*plpgsql_check_plugin_var_ptr)->error_callback;
+		prev_plpgsql_plugin->assign_expr = (*plpgsql_check_plugin_var_ptr)->assign_expr;
+
+#if PG_VERSION_NUM >= 150000
+
+		prev_plpgsql_plugin->assign_value = (*plpgsql_check_plugin_var_ptr)->assign_value;
+		prev_plpgsql_plugin->eval_datum = (*plpgsql_check_plugin_var_ptr)->eval_datum;
+		prev_plpgsql_plugin->cast_value = (*plpgsql_check_plugin_var_ptr)->cast_value;
+
+#endif
+
+		pinfo = init_profiler_info(pinfo, func);
+
+		PG_TRY();
+		{
+			if ((prev_plpgsql_plugin)->func_setup)
+				(prev_plpgsql_plugin->func_setup) (estate, func);
+
+			pinfo->prev_plugin_info = estate->plugin_info;
+			estate->plugin_info = pinfo;
+		}
+		PG_CATCH();
+		{
+			pinfo->prev_plugin_info = estate->plugin_info;
+			estate->plugin_info = pinfo;
+
+			PG_RE_THROW();
+		}
+		PG_END_TRY();
+	}
+
 	if (plpgsql_check_tracer)
 	{
 
@@ -2876,13 +2907,66 @@ plpgsql_check_profiler_func_init(PLpgSQL_execstate *estate, PLpgSQL_function *fu
 	}
 }
 
+void
+plpgsql_check_profiler_func_beg(PLpgSQL_execstate *estate, PLpgSQL_function *func)
+{
+	if (prev_plpgsql_plugin && prev_plpgsql_plugin->func_beg)
+	{
+		profiler_info *pinfo = estate->plugin_info;
+
+		PG_TRY();
+		{
+
+			Assert(pinfo);
+
+			estate->plugin_info = pinfo->prev_plugin_info;
+			(prev_plpgsql_plugin->func_beg) (estate, func);
+
+			pinfo->prev_plugin_info = estate->plugin_info;
+			estate->plugin_info = pinfo;
+		}
+		PG_CATCH();
+		{
+			pinfo->prev_plugin_info = estate->plugin_info;
+			estate->plugin_info = pinfo;
+
+			PG_RE_THROW();
+		}
+		PG_END_TRY();
+	}
+}
+
 void
 plpgsql_check_profiler_func_end(PLpgSQL_execstate *estate, PLpgSQL_function *func)
 {
 	profiler_info *pinfo = NULL;
 
 	if (estate)
+	{
 		pinfo = (profiler_info *) estate->plugin_info;
+
+		if (prev_plpgsql_plugin && prev_plpgsql_plugin->func_end)
+		{
+			PG_TRY();
+			{
+				Assert(pinfo);
+
+				estate->plugin_info = pinfo->prev_plugin_info;
+				(prev_plpgsql_plugin->func_end) (estate, func);
+
+				pinfo->prev_plugin_info = estate->plugin_info;
+				estate->plugin_info = pinfo;
+			}
+			PG_CATCH();
+			{
+				pinfo->prev_plugin_info = estate->plugin_info;
+				estate->plugin_info = pinfo;
+
+				PG_RE_THROW();
+			}
+			PG_END_TRY();
+		}
+	}
 	else if (top_pinfo)
 		pinfo = top_pinfo->pinfo;
 
@@ -2949,6 +3033,28 @@ plpgsql_check_profiler_stmt_beg(PLpgSQL_execstate *estate, PLpgSQL_stmt *stmt)
 {
 	profiler_info *pinfo = (profiler_info *) estate->plugin_info;
 
+	if (prev_plpgsql_plugin && prev_plpgsql_plugin->stmt_beg)
+	{
+		PG_TRY();
+		{
+			Assert(pinfo);
+
+			estate->plugin_info = pinfo->prev_plugin_info;
+			(prev_plpgsql_plugin->stmt_beg) (estate, stmt);
+
+			pinfo->prev_plugin_info = estate->plugin_info;
+			estate->plugin_info = pinfo;
+		}
+		PG_CATCH();
+		{
+			pinfo->prev_plugin_info = estate->plugin_info;
+			estate->plugin_info = pinfo;
+
+			PG_RE_THROW();
+		}
+		PG_END_TRY();
+	}
+
 	if (top_pinfo && top_pinfo->pinfo)
 	{
 		if (estate->eval_econtext != curr_eval_econtext)
@@ -3072,16 +3178,38 @@ plpgsql_check_profiler_stmt_end(PLpgSQL_execstate *estate, PLpgSQL_stmt *stmt)
 		Assert(top_pinfo && top_pinfo->pinfo);
 
 		pinfo = top_pinfo->pinfo;
-
-
 		estate = pinfo->estate;
 
 		is_error_stmt = top_pinfo->err_stmt == stmt;
 		cleaning_mode = true;
 	}
 	else
+	{
 		pinfo = (profiler_info *) estate->plugin_info;
 
+		if (prev_plpgsql_plugin && prev_plpgsql_plugin->stmt_end)
+		{
+			PG_TRY();
+			{
+				Assert(pinfo);
+
+				estate->plugin_info = pinfo->prev_plugin_info;
+				(prev_plpgsql_plugin->stmt_end) (estate, stmt);
+
+				pinfo->prev_plugin_info = estate->plugin_info;
+				estate->plugin_info = pinfo;
+			}
+			PG_CATCH();
+			{
+				pinfo->prev_plugin_info = estate->plugin_info;
+				estate->plugin_info = pinfo;
+
+				PG_RE_THROW();
+			}
+			PG_END_TRY();
+		}
+	}
+
 	if (top_pinfo && top_pinfo->pinfo && !cleaning_mode)
 	{
 		int		i;
@@ -3162,7 +3290,11 @@ plpgsql_check_profiler_stmt_end(PLpgSQL_execstate *estate, PLpgSQL_stmt *stmt)
 
 		Assert(pinfo->pi_magic == PI_MAGIC);
 
-		if (pstmt->queryid == NOQUERYID && estate)
+		/*
+		 * We can get query id only if stmt_end is not executed
+		 * in cleaning mode, because we need to execute expression
+		 */
+		if (pstmt->queryid == NOQUERYID && !cleaning_mode)
 			pstmt->queryid = profiler_get_queryid(estate, stmt,
 												  &pstmt->has_queryid,
 												  &pstmt->qparams);
@@ -3282,7 +3414,8 @@ plpgsql_check_needs_fmgr_hook(Oid fn_oid)
 		(*plpgsql_check_next_needs_fmgr_hook)(fn_oid))
 		return true;
 
-	if (!plpgsql_check_profiler)
+	if (!plpgsql_check_profiler &&
+		!plpgsql_check_tracer)
 		return false;
 
 	return plpgsql_check_is_plpgsql_function(fn_oid);
